@@ -10,7 +10,7 @@
 #' 
 #' If \code{coef = TRUE}, the coefficients are fixed as well.
 #' 
-#' @param x an object of class \code{seas}
+#' @param x an object of class \code{seas}.
 #' @param coef  logical, if \code{TRUE}, the coefficients are treated as fixed, 
 #'   instead of beeing estimated.
 #' @param test logical. By default the static call is executed and compared to 
@@ -84,19 +84,17 @@ static <- function(x, coef = FALSE, test = TRUE, verbose = FALSE){
   }
 
   z <- as.call(lc)
-
-  cat(deparse(z), sep = "\n")
   
   if (test){
     # testing the static call
     x.static <- eval(z, envir = globalenv())
-    test <- (all.equal(final(x.static), final(x), tolerance = 1e-05))
+    test <- (all.equal(log(final(x.static)), log(final(x)), tolerance = 1e-05))
     if (inherits(test, "character")){
-      stop(paste("Final Series of static and provided model differ.", test))
+      message(paste("Static series is different.", test))
     }
   }
 
-  invisible(z)
+  z
 }
 
 
